# Generated by roxygen2: do not edit by hand

S3method("[",glyrepr_structure)
S3method("[[<-",glyrepr_structure)
S3method(convert_to_generic,character)
S3method(convert_to_generic,glyrepr_composition)
S3method(convert_to_generic,glyrepr_structure)
S3method(count_mono,glyrepr_composition)
S3method(count_mono,glyrepr_structure)
S3method(format,glyrepr_composition)
S3method(format,glyrepr_structure)
S3method(get_mono_type,character)
S3method(get_mono_type,glyrepr_composition)
S3method(get_mono_type,glyrepr_structure)
S3method(is.na,glyrepr_composition)
S3method(is.na,glyrepr_structure)
S3method(obj_print_data,glyrepr_composition)
S3method(obj_print_data,glyrepr_structure)
S3method(obj_print_footer,glyrepr_structure)
S3method(pillar_shaft,glyrepr_composition)
S3method(pillar_shaft,glyrepr_structure)
S3method(vec_cast,character.glyrepr_composition)
S3method(vec_cast,character.glyrepr_structure)
S3method(vec_cast,glyrepr_composition.character)
S3method(vec_cast,glyrepr_composition.double)
S3method(vec_cast,glyrepr_composition.glyrepr_composition)
S3method(vec_cast,glyrepr_composition.glyrepr_structure)
S3method(vec_cast,glyrepr_composition.integer)
S3method(vec_cast,glyrepr_composition.list)
S3method(vec_cast,glyrepr_composition.logical)
S3method(vec_cast,glyrepr_structure.character)
S3method(vec_cast,glyrepr_structure.glyrepr_structure)
S3method(vec_cast,glyrepr_structure.igraph)
S3method(vec_cast,glyrepr_structure.list)
S3method(vec_cast,logical.glyrepr_composition)
S3method(vec_ptype2,glyrepr_composition.glyrepr_composition)
S3method(vec_ptype2,glyrepr_composition.logical)
S3method(vec_ptype2,glyrepr_structure.glyrepr_structure)
S3method(vec_ptype2,logical.glyrepr_composition)
S3method(vec_ptype_abbr,glyrepr_composition)
S3method(vec_ptype_abbr,glyrepr_structure)
S3method(vec_ptype_full,glyrepr_composition)
S3method(vec_ptype_full,glyrepr_structure)
S3method(vec_restore,glyrepr_composition)
S3method(vec_restore,glyrepr_structure)
export(as_glycan_composition)
export(as_glycan_structure)
export(available_monosaccharides)
export(available_substituents)
export(convert_to_generic)
export(count_mono)
export(fill_anomer_pos)
export(get_anomer)
export(get_anomer_pos)
export(get_mono_type)
export(get_structure_graphs)
export(get_structure_level)
export(glycan_composition)
export(glycan_structure)
export(has_linkages)
export(is_glycan_composition)
export(is_glycan_structure)
export(is_known_monosaccharide)
export(n_glycan_core)
export(o_glycan_core_1)
export(o_glycan_core_2)
export(possible_linkages)
export(reduce_structure_level)
export(remove_linkages)
export(remove_substituents)
export(severy)
export(simap)
export(simap_chr)
export(simap_dbl)
export(simap_int)
export(simap_lgl)
export(simap_structure)
export(simap_vec)
export(smap)
export(smap2)
export(smap2_chr)
export(smap2_dbl)
export(smap2_int)
export(smap2_lgl)
export(smap2_structure)
export(smap2_vec)
export(smap_chr)
export(smap_dbl)
export(smap_int)
export(smap_lgl)
export(smap_structure)
export(smap_unique)
export(smap_vec)
export(snone)
export(spmap)
export(spmap_chr)
export(spmap_dbl)
export(spmap_int)
export(spmap_lgl)
export(spmap_structure)
export(spmap_vec)
export(ssome)
export(structure_to_iupac)
export(valid_linkages)
import(vctrs)
importFrom(dplyr,.data)
importFrom(magrittr,"%>%")
importFrom(pillar,pillar_shaft)
