#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom dplyr .data
## usethis namespace: end
NULL

#' Internal vctrs methods
#'
#' @import vctrs
#' @keywords internal
#' @name glyrepr-vctrs
NULL
