% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/structure.R
\name{format_glycan_structure_subset}
\alias{format_glycan_structure_subset}
\title{Format a Subset of Glycan Structures with Optional Colors}
\usage{
format_glycan_structure_subset(x, indices, colored = TRUE)
}
\arguments{
\item{x}{A glyrepr_structure object}

\item{indices}{Indices of structures to format}

\item{colored}{A logical value indicating whether to add colors}
}
\value{
A character vector of formatted structures for the specified indices
}
\description{
Format a Subset of Glycan Structures with Optional Colors
}
\keyword{internal}
