% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glyrepr-package.R
\name{glyrepr-vctrs}
\alias{glyrepr-vctrs}
\title{Internal vctrs methods}
\description{
Internal vctrs methods
}
\keyword{internal}
