% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glyrepr-package.R
\docType{package}
\name{glyrepr-package}
\alias{glyrepr}
\alias{glyrepr-package}
\title{glyrepr: Representation for Glycan Compositions and Structures}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Computational representations of glycan compositions and structures, including details such as linkages, anomers, and substituents. Supports varying levels of monosaccharide specificity (e.g., "Hex" or "Gal") and ambiguous linkages. Provides robust parsing and generation of IUPAC-condensed structure strings. Optimized for vectorized operations on glycan structures, with efficient handling of duplications. As the cornerstone of the glycoverse ecosystem, this package delivers the foundational data structures that power glycomics and glycoproteomics analysis workflows.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://glycoverse.github.io/glyrepr/}
  \item \url{https://github.com/glycoverse/glyrepr}
  \item Report bugs at \url{https://github.com/glycoverse/glyrepr/issues}
}

}
\author{
\strong{Maintainer}: Bin Fu \email{23110220018@m.fudan.edu.cn} (\href{https://orcid.org/0000-0001-8567-2997}{ORCID}) [copyright holder]

}
\keyword{internal}
