% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/color-utils.R
\name{add_colors}
\alias{add_colors}
\title{Add Colors to Monosaccharides}
\usage{
add_colors(monos, colored = TRUE)
}
\arguments{
\item{monos}{A character vector of monosaccharide names}

\item{colored}{A logical value indicating whether to add colors}
}
\value{
A character vector with ANSI color codes
}
\description{
Add Colors to Monosaccharides
}
\keyword{internal}
