% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/color-utils.R
\name{colorize_iupac_string}
\alias{colorize_iupac_string}
\title{Apply Colors to IUPAC String (Monosaccharides + Gray Linkages)}
\usage{
colorize_iupac_string(iupac_text, mono_names)
}
\arguments{
\item{iupac_text}{Character string of IUPAC notation}

\item{mono_names}{Character vector of monosaccharide names to color}
}
\value{
Character string with colored monosaccharides and gray linkages
}
\description{
Apply Colors to IUPAC String (Monosaccharides + Gray Linkages)
}
\keyword{internal}
