% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/anomer.R
\name{get_anomer}
\alias{get_anomer}
\title{Get the Anomeric information}
\usage{
get_anomer(x)
}
\arguments{
\item{x}{A glycan structure vector (glyrepr_structure).}
}
\value{
a character vector of the anomeric information.
}
\description{
Get the Anomeric information
}
\examples{
x <- n_glycan_core()
get_anomer(x)
}
