% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linkage.R
\name{remove_linkages}
\alias{remove_linkages}
\title{Remove All Linkages from a Glycan}
\usage{
remove_linkages(glycan)
}
\arguments{
\item{glycan}{A glyrepr_structure vector.}
}
\value{
A glyrepr_structure vector with all linkages removed.
}
\description{
This function replaces all linkages in a glycan structure with "??-?",
as well as the reducing end anomer with "??-".
}
\examples{
glycan <- o_glycan_core_1(linkage = TRUE)
glycan
remove_linkages(glycan)

}
