% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/substituent.R
\name{remove_substituents}
\alias{remove_substituents}
\title{Remove All Substituents from a Glycan}
\usage{
remove_substituents(glycan)
}
\arguments{
\item{glycan}{A glyrepr_structure vector.}
}
\value{
A glyrepr_structure vector with all substituents removed.
}
\description{
This function replaces all substituents in a glycan structure with empty strings.
}
\examples{
(glycan <- o_glycan_core_1())
remove_substituents(glycan)

}
