% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/substituent.R
\name{available_substituents}
\alias{available_substituents}
\title{Available Substituents}
\usage{
available_substituents()
}
\value{
A character vector.
}
\description{
Get the available substituents for monosaccharides.
}
\examples{
available_substituents()

}
