% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/monosaccharide.R
\name{is_known_monosaccharide}
\alias{is_known_monosaccharide}
\title{Check if a Monosaccharide is Known}
\usage{
is_known_monosaccharide(mono)
}
\arguments{
\item{mono}{A character vector of monosaccharide names.}
}
\value{
A logical vector.
}
\description{
This function checks if a vector of monosaccharide names are known.
}
\examples{
is_known_monosaccharide(c("Gal", "Hex"))
is_known_monosaccharide(c("X", "Hx", "Nac"))

}
