% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/monosaccharide.R
\name{available_monosaccharides}
\alias{available_monosaccharides}
\title{Get Available Monosaacharides}
\usage{
available_monosaccharides(mono_type = "all")
}
\arguments{
\item{mono_type}{A character string specifying the type of monosaccharides.
Can be "all", "generic", or "concrete". Default is "all".}
}
\value{
A character vector of monosaccharide names.
}
\description{
This function returns a character vector of monosaccharide names of
the given type. See \code{\link[=get_mono_type]{get_mono_type()}} for monosaacharide types.
}
\examples{
available_monosaccharides()

}
