% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/color-utils.R
\name{add_gray_linkages}
\alias{add_gray_linkages}
\title{Add Gray Color to Linkages in IUPAC String}
\usage{
add_gray_linkages(iupac_text)
}
\arguments{
\item{iupac_text}{Character string of IUPAC notation}
}
\value{
Character string with linkages colored gray
}
\description{
Add Gray Color to Linkages in IUPAC String
}
\keyword{internal}
