% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/iupac-to-structure.R
\name{.parse_iupac_condensed_single}
\alias{.parse_iupac_condensed_single}
\title{Parse IUPAC-condensed string to glycan structure}
\usage{
.parse_iupac_condensed_single(x)
}
\arguments{
\item{x}{A single IUPAC-condensed string}
}
\value{
An igraph object representing the glycan structure
}
\description{
Internal functions for parsing IUPAC-condensed strings into igraph objects.
This supports the as_glycan_structure.character method.
}
\keyword{internal}
