% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/color-utils.R
\name{get_mono_color}
\alias{get_mono_color}
\title{Get Color for Concrete Monosaccharides}
\usage{
get_mono_color(mono)
}
\arguments{
\item{mono}{A monosaccharide name (character), potentially with substituents}
}
\value{
A color code (character)
}
\description{
Get Color for Concrete Monosaccharides
}
\keyword{internal}
